#include "minunit.h"
#include <assert.h>
#include <lcthw/bstrlib.h>
#include <lcthw/ringbuffer.h>

static RingBuffer *buffer;
static int DATA_LEN = 6;
static char *data[] = {"first" , "second", "third", "fourth" , "fifth", "sixth"};

char *test_create()
{
	buffer = RingBuffer_create(42);
	mu_assert(buffer != NULL, "Failed to create buffer.");

	return NULL;
}

char *test_read_write()
{
	int i;
	int rc;
	bstring datum;
	bstring target;

	// Set the data
	for (i = 0; i < DATA_LEN; i++)
	{
		rc = 0;
		datum = bfromcstr(data[i]);
		rc = RingBuffer_write(buffer, bdata(datum), blength(datum));
		mu_assert(rc == blength(datum), "Incorrect length returned on write");
	}

	// Get the data
	for (i = 0; i < DATA_LEN; i++)
	{
		rc = 0;
		datum = bfromcstr(data[i]);
		target = RingBuffer_gets(buffer, blength(datum));

		mu_assert(blength(target) == blength(datum), "Incorrect length returned on read");
		mu_assert(bstrcmp(datum, target) == 0, "Output not the same as input");
	}

	bdestroy(datum);
	bdestroy(target);

	return NULL;
}

char *test_destroy()
{
	RingBuffer_destroy(buffer);
	return NULL;
}

char *all_tests()
{
	mu_suite_start();

	mu_run_test(test_create);
	mu_run_test(test_read_write);
	mu_run_test(test_destroy);

	return NULL;
}

RUN_TESTS(all_tests)
